//************************************************************************
//*               ListFile : list file in directory V 1.2                *
//*----------------------------------------------------------------------*
//*                                                                      *
//*    Require : BipGDI Win V0.9.4                                       *
//*    Require : BipDate Win V0.9.2                                      *
//*    Require : BipNum Win V0.9.2                                       *
//*                                                                      *
//* Licence : NFM                                                        *
//************************************************************************


#pragma comment (lib, "comctl32.lib")


#include <windows.h>
#include <string>

// Include for Bip
#include "BipGDI.h"


	// Global variable
	string sDirName;
	const int MAXFILE = 1000;

	struct FILENFO
	{
		string sFileName;
		_int64 iFileSize;
		string sFileTime;
		bool bFileDir;
	};

	FILENFO FileNfo [MAXFILE];



//-------------------------------------------------------------
//!    Procedure for main window                              !
//-------------------------------------------------------------
LRESULT CALLBACK WndProc(HWND hBGCurWnd, UINT message, WPARAM wParam, LPARAM lParam)
{

	return DefWindowProc(hBGCurWnd, message, wParam, lParam);

}



//-------------------------------------------------------------
//!    Procedure for title window                             !
//-------------------------------------------------------------
LRESULT CALLBACK WndProcTiTle (HWND hBGCurWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
		case WM_PAINT:
		{

			// select window
			BG_WindowSelect ("title");

			// draw text on current window
			BG_TextFontSet ("Arial",20,1,1,1);
			BG_TextColorSet ("red");
			BG_TextDraw ("ListFile v 1.2 ");

			BG_TextFontSet ("Arial",20,1,0,0);
			BG_TextColorSet ("black");
			BG_TextDraw (" - ");
			BG_TextFontSet ("Arial",20,0,0,0);
			BG_TextColorSet ("magenta");
			BG_TextDraw (sDirName, "lf=true");

			// release current window
			BG_WindowRelease ();

			break;
		}

		return 0;

	}

	return DefWindowProc(hBGCurWnd, message, wParam, lParam);

}



//-------------------------------------------------------------
//!    Procedure for subtitle window                          !
//-------------------------------------------------------------
LRESULT CALLBACK WndProcSubTitle (HWND hBGCurWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
		case WM_PAINT:
		{

			// select window
			BG_WindowSelect ("subtitle");

			// draw text on current window
			BG_TextFontSet ("Arial",12, 1, 0, 0);
			BG_TextColorSet ("blue");
			BG_TextPositionSet (5);
			BG_TextDraw ("FileName");

			BG_TextPositionSet (500);
			BG_TextDraw ("Size");

			BG_TextPositionSet (600);
			BG_TextDraw ("Time");

			// release current window
			BG_WindowRelease ();

			break;
		}

		return 0;

	}

	return DefWindowProc(hBGCurWnd, message, wParam, lParam);

}



//-------------------------------------------------------------
//!    Procedure for file window                              !
//-------------------------------------------------------------
LRESULT CALLBACK WndProcFile (HWND hBGCurWnd, UINT message, WPARAM wParam, LPARAM lParam)
{

	int iError;

	switch (message)
	{
		case WM_PAINT:
		{
			int iLine;
			int iPoint;
			string sExt;

			// select window
			iError = BG_WindowSelect ("file");

			// display file info
			for (iLine = 0; iLine < MAXFILE; iLine++)
			{
				if (FileNfo [iLine].sFileName == "")
					break;

				iError = BG_TextFontSet ("Arial",10, 0, 0, 0);
				iError = BG_TextColorSet ("black");
				
				// directory in blue
				if (FileNfo [iLine].bFileDir == true)
					iError = BG_TextColorSet ("blue");

				// get extention of the file
				iPoint = FileNfo [iLine].sFileName.find ('.', 0);
				if (iPoint > 3)
				{
					sExt = FileNfo [iLine].sFileName.substr (iPoint + 1, 3);
					sExt = BG_StringToUpper (sExt);
				}
				else
					sExt = "";

				// exe in red
				if (sExt == "EXE")
					iError = BG_TextColorSet ("red");

				// dll in green
				if (sExt == "DLL")
					iError = BG_TextColorSet ("green");

				// bmp in magenta
				if (sExt == "BMP")
					iError = BG_TextColorSet ("magenta");

				// display name
				iError = BG_TextPositionSet (5);
				iError = BG_TextDraw (FileNfo [iLine].sFileName);

				// display size
				iError = BG_TextPositionSet (500);
				iError = BG_TextDraw (BG_IntToString (FileNfo [iLine].iFileSize));

				// display time
				iError = BG_TextPositionSet (600);
				iError = BG_TextDraw (FileNfo [iLine].sFileTime, "lf=true");

			}
			
			// release current window
			iError = BG_WindowRelease ();

			break;
		}

		return 0;
	}

	// automatic message
	iError = BG_WindowMessageAuto (message, wParam);

	return DefWindowProc (hBGCurWnd, message, wParam, lParam);

}



//-------------------------------------------------------------
//!    Format number with 2 char                              !
//-------------------------------------------------------------
string FormatInt (const int iInt)
{
	string sTemp;

	sTemp = BG_IntToString (iInt);

	if (sTemp.length() == 1)
		sTemp = "0" + sTemp;

	return sTemp;
}



//-------------------------------------------------------------
//!    Read File NFO                                          !
//-------------------------------------------------------------
int ReadFileNfo (const string sDirName)
{

	int iCpt;
	WIN32_FIND_DATA FindFileData;
	FILETIME ftUTC;
	SYSTEMTIME stLocal;
	HANDLE hFind;
	int iError;

	// reset file structure
	for (iCpt = 0; iCpt < MAXFILE; iCpt++)
	{
		FileNfo [iCpt].sFileName = "";
		FileNfo [iCpt].iFileSize = 0;
		FileNfo [iCpt].sFileTime = "";
		FileNfo [iCpt].bFileDir = false;
	}

	iCpt = 0;

	// get first file info
	hFind = FindFirstFile (sDirName.c_str(), &FindFileData);
	if (hFind == INVALID_HANDLE_VALUE) 
	{
		BG_MessageBox ("ListFile", "Error in Directory ");
		return -1;
	}

	// get all file info
	do
	{
		FileNfo [iCpt].sFileName = FindFileData.cFileName;
		FileNfo [iCpt].iFileSize = FindFileData.nFileSizeLow +
			(FindFileData.nFileSizeHigh * ((_int64)MAXDWORD + 1));

		// Convert the file creation time to local time.
		FileTimeToLocalFileTime (&FindFileData.ftLastWriteTime, &ftUTC);
		FileTimeToSystemTime (&ftUTC, &stLocal);

		FileNfo [iCpt].sFileTime = FormatInt (stLocal.wDay) + "/"
			+ FormatInt(stLocal.wMonth) + "/"
			+ FormatInt(stLocal.wYear) + " "
			+ FormatInt(stLocal.wHour) + ":"
			+ FormatInt(stLocal.wMinute);

		if (FindFileData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
		{
			FileNfo [iCpt].bFileDir = true;
			FileNfo [iCpt].sFileName += " (DIR)";
		}

		iCpt++;

		// get next file
		iError = FindNextFile (hFind, &FindFileData);

	}
	while (iError != 0 && iCpt < MAXFILE);


	// free handle
	FindClose (hFind);

	return 0;
}



//-------------------------------------------------------------
//!        BEGINING OF PROGRAM                                !
//-------------------------------------------------------------
int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{

	int iError;

	// open main window (800 X 600 pixels)
	iError = BG_WindowMainCreate ("Main", nCmdShow, WndProc, "bkcol=blue", 0, 0, 800, 600);
	if (iError != 0)
		return iError;

	// change main window title
	iError = BG_WindowTitleSet ("Main", "ListFile v1.2 , max " + BG_IntToString (MAXFILE)
								+ " files.");
	if (iError != 0)
		return iError;

	// directory to analyse
	sDirName = BG_InputBox ("Enter directory :", "", "C:\\WINNT\\*.*");
	if (BG_InputBoxErrorGet () != 0)
		return BG_InputBoxErrorGet ();

	// Read File NFO
	iError = ReadFileNfo (sDirName);
	if (iError != 0)
		return iError;

	// create title window with no border
	iError = BG_WindowChildCreate ("title", WndProcTiTle, "bkcol=grey", 100, 5, 600, 43);
	if (iError != 0)
		return iError;

	// create subtitle window with border
	iError = BG_WindowChildCreate ("subtitle", WndProcSubTitle,
									"bkcol=yellow;border=true", 0, 52, 0, 20);
	if (iError != 0)
		return iError;

	// create file window with no border
	iError = BG_WindowChildCreate ("file", WndProcFile, "bkcol=grey", 0, 77, 0, 495);
	if (iError != 0)
		return iError;


	// process message loop
	return BG_MessageLoop ();

}


